use std::io;
use std::time::{Duration, Instant};

use crossterm::event;
use crossterm::event::{Event, KeyCode};
use tui::backend::Backend;
use tui::style::*;
use tui::text::*;
use tui::widgets::*;
use tui::{Frame, Terminal};

use sea_battle_backend::data::*;

use crate::constants::*;
use crate::ui_screens::utils::centered_rect_size;
use crate::ui_screens::ScreenResult;
use crate::ui_widgets::game_map_widget::GameMapWidget;

struct SetBotsLayoutScreen {
    curr_boat: usize,
    layout: BoatsLayout,
}

pub fn set_boat_layout<B: Backend>(
    rules: &GameRules,
    terminal: &mut Terminal<B>,
) -> io::Result<ScreenResult<BoatsLayout>> {
    let mut model = SetBotsLayoutScreen {
        curr_boat: 0,
        layout: BoatsLayout::gen_random_for_rules(rules)
            .expect("Failed to generate initial boats layout"),
    };

    let mut last_tick = Instant::now();
    loop {
        terminal.draw(|f| ui(f, &mut model, rules))?;

        let timeout = TICK_RATE
            .checked_sub(last_tick.elapsed())
            .unwrap_or_else(|| Duration::from_secs(0));

        if event::poll(timeout)? {
            if let Event::Key(key) = event::read()? {
                match key.code {
                    KeyCode::Char('q') => return Ok(ScreenResult::Canceled),
                    KeyCode::Enter => {
                        if model.layout.is_valid(rules) {
                            return Ok(ScreenResult::Ok(model.layout));
                        }
                    }
                    KeyCode::Down => model.curr_boat -= 1,
                    KeyCode::Up => model.curr_boat += model.layout.number_of_boats() - 1,
                    _ => {}
                }

                model.curr_boat %= model.layout.number_of_boats();
            }
        }
        if last_tick.elapsed() >= TICK_RATE {
            last_tick = Instant::now();
        }
    }
}

fn ui<B: Backend>(f: &mut Frame<B>, model: &mut SetBotsLayoutScreen, rules: &GameRules) {
    let area = centered_rect_size(
        rules.map_width as u16 + 10,
        rules.map_height as u16 + 10,
        f.size(),
    );

    let game_map_widget = GameMapWidget::new(rules);

    f.render_widget(game_map_widget, area);
}
